; Run the markov chain on the given cluster (in markov form), for
; start * 10^iters iterations. Each row in the returned array contains the
; contents of the "histogram" after start * 10^i iterations (where i is the row
; index).
function markov_progressive,points,perim_n,mem,start,iters
    cluster_size = n_elements(points)
    trend = ulon64arr(cluster_size*2l + 2l, iters, /nozero)

    ; First row; "start" iterations
    trend[*, 0] = markov_interruptible(points,perim_n,mem, 1, start)

    ; Additional rows: when we reach any given row (with index i), we already
    ; have start*10^(i-1) iterations. To reach start*10^i iterations, we must
    ; add start*10^(i-1)*9 iterations.
    multiplier = 9ull
    for i=1,iters-1 do begin
       ; Start with the data from the iterations we have already done
       trend[*, i] = trend[*, i-1]
       ; Add the iterations we must do to reach the next "milestone". Note that
       ; "multiplier" equals 10^(i-1)*9.
       trend[*, i] += markov_interruptible(points,perim_n,mem, multiplier, start)
       multiplier *= 10ull
    endfor

    return, trend
end