; Generate a random number by the "mzran" algorithm in Marsiglia and Zaman
; ("MZ"), Computers in Physics, 8, 107 (1994)
function randint
    common rand_seed, ir1, jr1, kr1, r2

    ;rand_seed_init

    ; Advance by one in MZ's "sequence 1"
    random = ir1 - kr1
    if random LT 0l then random += 2147483579l
    ir1 = jr1 & jr1 = kr1 & kr1 = random

    ; Advance by one in MZ's "sequence 6"
    r2 = r2*69069l + 1013904243l

    ; Add the new numbers in each sequence together (thus combining the two generators) and return
    random += r2
    return, random
end

function rand
    return, 0.5D + double(randint())*0.23283064D-9
end

function rand_seed_undefined
    common rand_seed
    return, N_ELEMENTS(ir1) EQ 0 || N_ELEMENTS(jr1) EQ 0 || N_ELEMENTS(kr1) EQ 0 || N_ELEMENTS(r2) EQ 0
end

pro rand_seed_init
    common rand_seed

    ; If the seed is already initalized, then we are done
    if ~rand_seed_undefined() then return

    catch, restore_error
    if restore_error EQ 0 then begin
       restore, 'mzrand.sav'
       ir1 = long(ir1)
       jr1 = long(jr1)
       kr1 = long(kr1)
       r2 = long(r2)
    endif else begin
       ; Setup a default seed (some arbitrary numbers)
       ir1 = 28167234l
       jr1 = 86219809l
       kr1 = 68146984l
       r2 = 54465289l
    endelse

end

pro rand_seed_save
    common rand_seed
    save, FILENAME = 'mzrand.sav', ir1, jr1, kr1, r2
end