; Given a set of points (with one point on the origin), determine if that set
; of points represent one (closed) cluster.
; Note that mem *must* be clear (i.e. trace_cluster_memclear must be run on mem
; between each call to this function).
function trace_cluster,points,points_n,start,mem
    mem.map[points[0:points_n-1]] = mem.map_occupied

    ; "Notice" the starting point (we just assume that it is in the cluster).
    mem.map[start] = mem.map_noticed
    mem.todo[mem.todo_n] = start
    mem.todo_n++

    ; While we still have points to deal with ...
    while mem.todo_n GT 0l do begin
       ; Consider the point on the top of the todo stack. Remove it
       mem.todo_n--
       point = mem.todo[mem.todo_n]

       ; Make sure its neighbors are "noticed".
       adjacent_points = [point - mem.map_dim, point - 1l, point + 1l, point + mem.map_dim]
       for i = 0, 3 do begin
         cur_adj_pt = adjacent_points[i]

         ; If this neighbor hasn't been noticed, we need to fix that.
         if mem.map[cur_adj_pt] NE mem.map_noticed then begin
          ; If the neighbor is occupied ...
          if mem.map[cur_adj_pt] EQ mem.map_occupied then begin
           ; ... then it belongs in todo[].
           mem.todo[mem.todo_n] = cur_adj_pt
           mem.todo_n++
          endif else begin
           ; Otherwise, it's unoccupied, so it belongs in perim[].
           mem.perim[mem.perim_n] = cur_adj_pt
           mem.perim_n++
          endelse
          ; Mark the neighbor as "noted"
          mem.map[cur_adj_pt] = mem.map_noticed
         endif
       endfor

       ; Now that all of its neighbors have been noticed, the point now
       ; qualifies to be counted in cluster_n
       mem.cluster_n++

    endwhile

    ; Now, all points in the cluster that covers the origin are counted in
    ; cluster_n. If the number of points in this cluster is equal to the number
    ; of points given, we have a single cluster.
    return, (mem.cluster_n EQ points_n)

end