; Given a cluster represented in coordinate form (i.e. as returned by
; cluster_init), return a structure that represents the cluster and that is
; usable by markov_chain and friends. Note that this structure also contains
; pre-allocated memory for these functions, so the resulting structure could be
; very large.
function bi2markov,cluster_bi
    ; "Parameters" for @trace_cluster_mem
    points_n = n_bi_points(cluster_bi.sites)
    max_coord = points_n - 1U

    mem = { $
       ; **********************
       ; Markov Chain variables:

       ; points: the list of all points in the cluster, represented in uni form
       points: bi2uni(cluster_bi.sites, points_n*2l + 1l, (points_n*2l + 1l)/2), $

       ; perim_n: the number of perimeter points in the cluster represented in points[]
       perim_n: n_bi_points(cluster_bi.perim), $


       ; ************************
       ; trace_cluster variables:

       ; The cluster sites counter. Holds the number of points that are
       ; connected to the starting point and that are not adjacent to any
       ; "unnoticed" points.
       trace_cluster_n:0l, $

       ; The "todo" stack. This contains all occupied points that are adjacent
       ; to some point(s) counted in trace_cluster_n but might be adjacent to
       ; some points we haven't noticed yet. Points in this stack are removed
       ; here (and then counted in cluster_n) once we check for any unnoticed
       ; neighbors.
       trace_todo:lonarr(points_n, /NOZERO), $
       trace_todo_n:0l, $

       ; The perimeter sites stack. This contains all occupied points that are
       ; adjacent to some point(s) counted in trace_cluster_n but are no
       ; themselves occupied. When all cluster points have been counted, this
       ; stack will contain the cluster's perimeter points
       trace_perim:lonarr(2l*points_n + 2l, /NOZERO), $
       trace_perim_n:0l, $

       ; The "map": a square array of dimension map_dim. Each element in this
       ; array corresponds to a position. A value equal to trace_map_occupied
       ; means that the location occupied but is not in the trace_todo or
       ; trace_perim arrays nor has been counted in trace_cluster_n (i.e. it
       ; has not been "noticed"). A value equal to trace_map_noticed that the
       ; location is "noticed" and is either occupied or is perimeter site. Any
       ; other value means that the location is either unoccupied or an
       ; unnoticed perimeter site. Note that we use variables rather than fixed
       ; constants for these states to facilitate fast "clearing" of this map.
       trace_map:uintarr(max_coord*2l + 3l, max_coord*2l + 3l), $
       trace_map_dim:max_coord*2l + 3l, $
       trace_map_occupied:1U, $
       trace_map_noticed:2U $
    }
    return, mem
end