; Computes the average perimeter length encountered when running the markov 
; chain with memory structure "mem" for "sub_iterations" iterations. Performs
; this operation "iterations" times. Returns an array containing these averages
; in the order in which they were computed. If sub_iterations = 1, then this
; function effectively tracks perimeter length over time.

function equilibrium_watch,mem,iterations,sub_iterations
    ; Make sub_iterations an optional argument
    if n_elements(sub_iterations) EQ 0 then sub_iterations = 1
    perim_trend = intarr(iterations)
    n_points = n_elements(mem.points)
    hist_multiplier = indgen(2*n_points+2)
    for i=0,iterations-1 do begin
       hist = ulon64arr(2l*n_points + 2l)
       markov_chain, mem, sub_iterations, hist
       perim_trend[i] = fix(total(hist*hist_multiplier)/total(hist))
    endfor

    return, perim_trend
end
