; Given a memory block representing a cluster and containing reserved temporary
; memory, run the markov chain for the given number of iterations and add the
; resulting frequency histogram to perim_frequency.
pro markov_chain,mem,iterations,perim_frequency
    points_n = n_elements(mem.points)
    center = bi2uni([0, 0], mem.trace_map_dim, mem.trace_map_dim/2)

    for j=1ull,iterations do begin
       ; Randomly select a non-center point in points[] ...
       sel_index = long(mzrand(1)*(points_n-1))
       if mem.points[sel_index] EQ center then sel_index++
       ; ... and move it to the end of the list.
       sel_point = mem.points[sel_index]
       mem.points[sel_index] = mem.points[points_n-1]
       mem.points[points_n-1] = sel_point
       ; Now let's consider the "intermediate cluster", the cluster that
       ; results from removing sel_point from points[]. See if the intermediate
       ; cluster is closed. By passing points_n-1 as the length of points[], we
       ; cause trace_cluster to ignore sel_point and thus look at the
       ; intermediate cluster.
       if trace_cluster(points_n-1, center, mem) then begin
         ; If it does remain closed, the Markov Chain calls for us to remove
         ; the point (getting the intermediate cluster) and then to add a
         ; random point to the cluster on it's perimeter. We already have the
         ; perimeter of the intermeidate cluster in map.perim, so we can
         ; simulate this process by just replacing sel_point with a random
         ; point from mem.perim.
         new_point = mem.trace_perim[long(mzrand(1)*(mem.perim_n))]
         mem.points[points_n-1] = new_point
         ; Now we need to find the new perimeter length. In adding a new point
         ; to the "intermediate cluster", we lose the perimeter site that has
         ; become a cluster site, but the new site may have gained us some new
         ; perimeter sites. If any locations adjacent to new_point were not
         ; "noticed" by trace_cluster, that means that they were neither
         ; cluster locations nor perimeter sites of the intermediate cluster.
         ; This means that these sites became new perimeter sites with the
         ; addition of new_point. The number of such sites should be added to
         ; perim_n.
         adjacent_points = find_adjacent(new_point, mem.trace_map_dim)
         perim_new = total(mem.trace_map[adjacent_points] NE mem.trace_map_noticed, /INTEGER)
         mem.perim_n = mem.trace_perim_n - 1 + perim_new
       endif
       ; If the cluster was not closed, we leave the points[] and perim_n
       ; unchanged.

       ; Make a record of the perimeter of whatever we came up with.
       perim_frequency[mem.perim_n-1]++
    endfor
end
