; Workaround an IDL bug so that a long markov_chain run is breakable under
; Windows, but do so in a way that does not degrade performance too much.
; See http://www.dfanning.com/misc_tips/endlessloop.html for information about
; the bug and the workaround.
; Parameters:
;  - mem: the memory and data structure for markov_chain and friends, as generated by bi2markov
;  - iterations: the number of markov chain iterations to perform
;  - sub_iterations: obsolete. Don't worry about it.
; TODO:
;   - Automatically adjust the number of iterations between Widget_Event calls
;     so that the period between Ctrl+Break checks is reasonable, regardless of
;     how long a markov_chain iteration takes.

function markov_interruptible,mem,iterations,sub_iterations
    if n_elements(sub_iterations) EQ 1 then begin
       ; Backwards compatibility for an older version of this function that
       ; demanded that a stepping be provided explicitly.
       max_run = sub_iterations
       iters_todo = iterations*sub_iterations
    endif else begin
       max_run = 1000ul
       iters_todo = iterations
    endelse

    perim_frequency = ulon64arr(2l*n_elements(mem.points) + 2l)
    
    ; If we need the workarround
    if !VERSION.OS EQ "Win32" then begin
        while iters_todo GE max_run do begin
           markov_chain,mem,max_run,perim_frequency
           iters_todo -= max_run
           ; The workaround (I hate IDL):
           tmp = Widget_Event(/NoWait)
        endwhile
        
        ; We should do this again to guard against the case where this function is
        ; repeatedly called for small numbers of iterations.
        tmp = Widget_Event(/NoWait)
    endif

    ; Take care of any leftovers
    ; If we didn't use the workarround, we will perform *all* of the iterations 
    ; at this step
    if iters_todo GT 0 then markov_chain,mem,iters_todo,perim_frequency

    return, perim_frequency
end
