; Determines if the cluster represented by the first points_n points in
; mem.points is closed. As a side effect, mem.trace_perim becomes the list of
; perimeter points in the cluster connected to start, mem.trace_perim_n
; becomes the length of this list, and all perimeter sites and cluster sites
; are marked as mem.trace_map_noticed in mem.trace_map.

function trace_cluster,points_n,start,mem
    ; First, clear the memory.
    mem.trace_cluster_n = 0l
    mem.trace_todo_n = 0l
    mem.trace_perim_n = 0l
    ; If map_occupied and map_noticed are about to overflow, then zero the map
    ; and set map_occupied and map_noticed back to default values.
    if mem.trace_map_noticed EQ 65534U then begin
       ; IDL has decreed that mem.map is not a "named variable", and so I can't
       ; use replicate_inplace to zero the map. I hate IDL.
       ; replicate_inplace, mem.map, 0U
       mem.trace_map[*] = 0U
       mem.trace_map_occupied = 1U
       mem.trace_map_noticed = 2U
    endif else begin
       ; Otherwise, we can effect a "quick" resetting by incrementing
       ; map_occupied and map_noticed to values that are greater than any
       ; possible value of any map[] element.
       mem.trace_map_occupied += 2U
       mem.trace_map_noticed += 2U
    endelse

    mem.trace_map[mem.points[0:points_n-1]] = mem.trace_map_occupied

    ; "Notice" the starting point (we just assume that it is in the cluster).
    mem.trace_map[start] = mem.trace_map_noticed
    mem.trace_todo[mem.trace_todo_n] = start
    mem.trace_todo_n++

    ; While we still have points to deal with ...
    while mem.trace_todo_n GT 0l do begin
       ; Consider the point on the top of the todo stack. Remove it
       mem.trace_todo_n--
       point = mem.trace_todo[mem.trace_todo_n]

       ; Make sure its neighbors are "noticed".
       adjacent_points = [point - mem.trace_map_dim, point - 1l, point + 1l, point + mem.trace_map_dim]
       for i = 0, 3 do begin
         cur_adj_pt = adjacent_points[i]

         ; If this neighbor hasn't been noticed, we need to fix that.
         if mem.trace_map[cur_adj_pt] NE mem.trace_map_noticed then begin
          ; If the neighbor is occupied ...
          if mem.trace_map[cur_adj_pt] EQ mem.trace_map_occupied then begin
           ; ... then it belongs in todo[].
           mem.trace_todo[mem.trace_todo_n] = cur_adj_pt
           mem.trace_todo_n++
          endif else begin
           ; Otherwise, it's unoccupied, so it belongs in perim[].
           mem.trace_perim[mem.trace_perim_n] = cur_adj_pt
           mem.trace_perim_n++
          endelse
          ; Mark the neighbor as "noted"
          mem.trace_map[cur_adj_pt] = mem.trace_map_noticed
         endif
       endfor

       ; Now that all of its neighbors have been noticed, the point now
       ; qualifies to be counted in cluster_n
       mem.trace_cluster_n++

    endwhile

    ; Now, all points in the cluster that covers the origin are counted in
    ; cluster_n. If the number of points in this cluster is equal to the number
    ; of points given, we have a single cluster.
    return, (mem.trace_cluster_n EQ points_n)

end